/**
 # connect-four.js
 # Connect4 game -Creates/provides interfaces to play game 
 #
 # @author Joachim Schmidt - joachim.schmidt@jschmidt-systemberatung.de
 # @copyright Copyright (C) 2022 Joachim Schmidt. All rights reserved.
 # @license	 http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 #
 # change activity:
 */
 
function Game() {
    this.rows = 6; // Height
    this.columns = 7; // Width
    this.status = 0; // 0: running, 1: won, 2: lost, 3: tie
    this.depth = 5; // Search depth
    this.score = 100000, // Win/loss score
    this.round = 0; // 0: Human, 1: Computer
    this.winning_array = []; // Winning (chips) array
    this.iterations = 0; // Iteration count
	this.draws = 0; // nummber of draws
	this.moves = [];
	this.winmarked = false;
	//this.resetCheckboard();
    
    that = this;
    that.init();
    jQuery("#undo-move").hide();
}

Game.prototype.init = function() {
    // Generate 'real' board
    // Create 2-dimensional array
    var game_board = new Array(that.rows);
    for (var i = 0; i < game_board.length; i++) {
        game_board[i] = new Array(that.columns);

        for (var j = 0; j < game_board[i].length; j++) {
            game_board[i][j] = null;
        }
    }

    // Create from board object (see board.js)
    this.board = new Board(this, game_board, 0);

    // Generate visual board
    var game_board = '<col/><col/><col/><col/><col/><col/><col/>';

	
    for (var i = 0; i < that.rows; i++) {
        game_board += "<tr>";
        for (var j = 0; j < that.columns; j++) {
            game_board += "<td class='empty'></td>";
        }
        game_board += "</tr>";
    }

    document.getElementById('game_board').innerHTML = game_board;

    // Action listeners
    var td = document.getElementById('game_board').getElementsByTagName("td");

    for (var i = 0; i < td.length; i++) {
        if (td[i].addEventListener) {
            td[i].addEventListener('click', that.act, false);
        } else if (td[i].attachEvent) {
            td[i].attachEvent('click', that.act)
        }
    }
    
    that.startMessage();
}

/**
 * On-click event
 */
Game.prototype.act = function(e) {
    var element = e.target || window.event.srcElement;

    // Check if not in animation and start with human
    if (!(jQuery('#coin').is(":animated"))) {
        if (that.round == 0) {
            that.place(element.cellIndex);
        }
    }
}

Game.prototype.startMessage = function() {
	
    var html = document.getElementById('status');
    var game_started = "Game started ";
    if  (document.getElementById('lang').lang == "de")
	  game_started = "Spiel gestartet";  
    html.className = "status-running";
    html.innerHTML = game_started;	
}	

/**
 * Place coin
 */
Game.prototype.place = function(column) {
    // If not finished
    if (that.board.score() != that.score && that.board.score() != -that.score && !that.board.isFull()) {
		
		var move = new Array();
		
        for (var y = that.rows - 1; y >= 0; y--) {
            if (document.getElementById('game_board').rows[y].cells[column].className == 'empty') {
                if (that.round == 1) {
                    var coin_x = column * 51;
                    var coin_y = y * 51;
                    jQuery('#coin').attr('class', 'cpu-coin').css({'left': coin_x}).fadeIn('fast').animate({'top': coin_y + 'px'}, 700, 'easeOutBounce', function() {
                        document.getElementById('game_board').rows[y].cells[column].className = 'coin cpu-coin';
                        jQuery('#coin').hide().css({'top': '0px'});
                        
                        if (!that.board.place(column)) {
                            return alert("Invalid move1! " + 'row = ' +y +'column = ' +column);
                        }

                        that.round = that.switchRound(that.round);
                        move = [y, column];	
	                    that.moves.push(move);                        
                        that.updateStatus();
                    });
                } else {	                
                    var coin_x = column * 51;
                    var coin_y = y * 51;
                    jQuery('#coin').attr('class', 'human-coin').css({'left': coin_x}).fadeIn('fast').animate({'top': coin_y + 'px'}, 700, 'easeOutBounce', function() {
                        document.getElementById('game_board').rows[y].cells[column].className = 'coin human-coin';
                        jQuery('#coin').hide().css({'top': '0px'});
                        that.generateComputerDecision();
                        
                        if (!that.board.place(column)) {
                             return alert("Invalid move2! " + 'row = ' +y +' column = ' +column);
                        }

                        that.round = that.switchRound(that.round);
                        move = [y, column];	
	                    that.moves.push(move);    
                        that.updateStatus();
                    });
                }
                break;
            }
        }
    }
    
}

Game.prototype.undoMove = function() {
	
	if (that.moves.length < 1)
	 return;

   for (var i=0; i<2; i++)
   {
	 var cell = that.moves.pop(); 
	 var y = cell[0];
	 var column = cell[1]; 
	 document.getElementById('game_board').rows[y].cells[column].className = 'empty'
	 this.board.field[y][column] = null;
   } 

    if (that.winmarked)
      that.unmarkWin();
	that.updateCheckboard();
	that.updateStatus();
}	

Game.prototype.generateComputerDecision = function() {
    if (that.board.score() != that.score && that.board.score() != -that.score && !that.board.isFull()) {

        //document.getElementById('loading').style.display = "block"; // Loading message
        // AI is thinking
        setTimeout(function() {
            // Debug time
            // var startzeit = new Date().getTime();

            var difficulty = document.getElementById('difficulty');
            maxDepth = difficulty.options[difficulty.selectedIndex].value;         	
           	           
	        var ai_move = aiMove();  
	        that.updateCheckboard(); 	  
		    that.draws++; 
		
            // var laufzeit = new Date().getTime() - startzeit;
            that.place(ai_move);
           // document.getElementById('loading').style.display = "none"; // Remove loading message
        }, 100);
    }
}


Game.prototype.switchRound = function(round) {
    // 0 Human, 1 Computer
    if (round == 0) {
        return 1;
    } else {
        return 0;
    }
}

Game.prototype.updateStatus = function() {
    
	that.status = 0;
	// Tie
    if (that.board.isFull()) {
        that.status = 3;
      //  alert("Tie!");
    } 
	
	// Human won
    if (that.board.score() == -that.score) {
        that.status = 1;
        that.markWin();
      //  alert("You have won!");
    }

    // Computer won
    if (that.board.score() == that.score) {
        that.status = 2;
	    that.markWin(); 
       // alert("You have lost!");
    }

    var game_won = "you have won";
	var game_lost = "you have lost";
	var game_tie = "it was a draw";
	var game_runs = "Game running";
		
    if  (document.getElementById('lang').lang == "de")
	{
		game_won = "du hast gewonnen";
		game_lost = "du hast verloren";
		game_tie = "unentschieden";
		game_runs = "Spiel l&auml;uft";
	}
			
    var html = document.getElementById('status');
    if (that.status == 0) {
        html.className = "status-running";
        html.innerHTML = game_runs;
        that.updateCheckboard();
        if (that.moves.length > 1)
	       jQuery("#undo-move").show();
	    else
	      jQuery("#undo-move").hide();    
    } else if (that.status == 1) {
        html.className = "status-won";
        html.innerHTML = game_won;
        this.moves = []; 
        jQuery("#undo-move").hide();
    } else if (that.status == 2) {
        html.className = "status-lost";
        html.innerHTML = game_lost;
    } else {
        html.className = "status-tie";
        html.innerHTML = game_tie;
    }
}

Game.prototype.unmarkWin = function() {
	
   for (var i=0; i<6; i++)
	{
	 for (var j=0; j<7; j++)
	  {
	    if ( document.getElementById('game_board').rows[i].cells[j].className == 'coin cpu-coin win' )
	      document.getElementById('game_board').rows[i].cells[j].className = 'coin cpu-coin';
	   }
	 } 
	 that.winmarked = false; 
}	

Game.prototype.markWin = function() {
    for (var i = 0; i < that.winning_array.length; i++) {
        var name = document.getElementById('game_board').rows[that.winning_array[i][0]].cells[that.winning_array[i][1]].className;
        document.getElementById('game_board').rows[that.winning_array[i][0]].cells[that.winning_array[i][1]].className = name + " win";
    }
    that.winmarked = true;
}

Game.prototype.restartGame = function() {
    //if (confirm('Game is going to be restarted.\nAre you sure?')) {
        // Dropdown value
        var difficulty = document.getElementById('difficulty');
        var depth = difficulty.options[difficulty.selectedIndex].value;
        that.depth = depth;
        maxDepth = depth;
		that.status = 0;
        that.round = 0;
        that.init();
		that.draws = 0;
		that.moves = []; 
        that.resetCheckboard();
        that.updateStatus();
        that.startMessage();
        jQuery("#undo-move").hide();

        // Re-assign hover
        jQuery('td').hover(function() {
            jQuery(this).parents('table').find('col:eq('+jQuery(this).index()+')').toggleClass('hover');
        });
    //}
}

Game.prototype.logCheckboard = function() 
{
	for (var i=0; i<6; i++)
	{
		console.log( checkboard[i] );
	} 	   
}

Game.prototype.resetCheckboard = function() 
{
	for (var i=0; i<6; i++)
	{
	 for (var j=0; j<7; j++)
	 {
	   checkboard[i][j] = ''; 
	  }
	   //console.log('row: ' + i+ ' = ' + checkboard[i] );
	} 
	   
}	

Game.prototype.updateCheckboard = function() 
{
   	for (var i=0; i<6; i++)
	{
	 for (var j=0; j<7; j++)
	  {
		//console.log('\n' + document.getElementById('game_board').rows[i].cells[j].className);
	   if ( document.getElementById('game_board').rows[i].cells[j].className == 'coin cpu-coin' )
	    checkboard[i][j] = 'b';
	   else if ( document.getElementById('game_board').rows[i].cells[j].className == 'coin human-coin' )
	    checkboard[i][j] = 'r';
	   else
	     checkboard[i][j] = ''; 
	  }
	} 

}
/**
 * Start game
 */
function Start() {
    window.Game = new Game();

    // Hover background, now using jQuery
    jQuery('td').hover(function() {
        jQuery(this).parents('table').find('col:eq('+jQuery(this).index()+')').toggleClass('hover');
    });
}

window.onload = function() {
    Start()
};
